package gomoku;

import gomoku.Game.GameStatus;
import gomoku.Game.SquareState;

import java.io.IOException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.concurrent.Task;
import javafx.concurrent.WorkerStateEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.Label;
import javafx.scene.layout.GridPane;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;

public class GomokuController {
	
	/**
	 * Used as choice box item.
	 * (Currently not used)
	 * @author Ago
	 *
	 */
	private class ChoiceItem {
		private final String value;
		private final int key;
		public ChoiceItem(int key, String value) {
			this.key = key;
			this.value = value;
		}
		public int getKey() { return key; }
		public String toString() { return value; }
	}

	private final ExecutorService executorService ;
	private Game game;
	private Square[][] squares;
	
	@FXML
	private ChoiceBox<String> playersChoiceBox;
	
	public GomokuController(Game game) {
		this.game = game;
		squares = new Square[game.getHeight()][game.getWidth()];


		executorService = Executors.newSingleThreadExecutor(new ThreadFactory() {
			@Override
			public Thread newThread(Runnable r) {
				Thread thread = new Thread(r);
				thread.setDaemon(true);
				return thread;
			}
		});
	}
	
	@FXML
	private GridPane board;
	
	@FXML
	private Label statusLabel;
	
	@FXML
	private Label playerLabel;
	
	
	public void initialize() throws IOException {
		setUpSquares();
		initializePlayerSelection();
		initializeGameStatusListener();
		
		if (playersChoiceBox.getItems().size() == 0) {
			ObservableList<String> choices = FXCollections.observableArrayList();
			choices.add("Human VS StudentStrategy");
			choices.add("StudentStrategy VS Human");
			playersChoiceBox.setItems(choices);
			playersChoiceBox.getSelectionModel().select(0);
		}
		
	}
	
	private void initializeGameStatusListener() {
		game.gameStatusProperty().addListener((event) -> 
		{
			
			GameStatus status = game.gameStatusProperty().get();
			if (status == GameStatus.BLACK_WON || status == GameStatus.WHITE_WON) {
				statusLabel.fontProperty().set(Font.font(null, FontWeight.BOLD, 12));
			}
			statusLabel.setText(status.toString());
		});
	}
	private void initializePlayerSelection() {
		game.currentPlayerProperty().addListener((event) ->
		{
			Player player = game.currentPlayerProperty().get();
			playerLabel.setText(player.getSquareState() + " (" + player.getName() + ") to move");
			player.getSquareState();
			if (player instanceof ComputerPlayer) {
				final Task<Location> computerMoveTask = new Task<Location>() {
					@Override
					public Location call() throws Exception {
						return ((ComputerPlayer)player).getMove(game.getSimpleBoard(),
								player.getSquareState() == SquareState.WHITE ? 
										SimpleBoard.PLAYER_WHITE :
										SimpleBoard.PLAYER_BLACK);
					}
				};
				computerMoveTask.setOnSucceeded(new EventHandler<WorkerStateEvent>() {
					@Override
					public void handle(WorkerStateEvent event) {
						game.makeMove(player, computerMoveTask.getValue());
					}
				});
				executorService.submit(computerMoveTask);
			}
		});
	}
	
	private void setUpSquares() throws IOException {
		for (int row = 0; row < game.getHeight(); row++) {
			for (int col = 0; col < game.getWidth(); col++) {
				final Square square = new Square(row, col, game);
				board.getChildren().add(square);
				squares[row][col] = square;
			}
		}
	}
	public void setBoardGridPane(GridPane boardGridPane) {
		this.board = boardGridPane;
	}
	
	@FXML
	public void newGame() throws IOException {
		if (board != null) {
			board.getChildren().clear();
		}
		String playersText = (String) this.playersChoiceBox.getSelectionModel().getSelectedItem();
		String[] playerNames = playersText.split(" VS ");
		Player[] players = new Player[2];
		
		for (int i = 0; i < 2; i++) {
			if (playerNames[i].equals("Human")) {
				players[i] = new Player("Human");
			} else if (playerNames[i].equals("StudentStrategy")) {
				players[i] = new ComputerPlayer(new StudentStrategy());
			}
		}
		Player playerW = players[0];
		Player playerB = players[1];
		//Player playerO = new Player("human");
		//Player playerO = new ComputerPlayer(new DummyStrategy());
		//Player playerO = new ComputerPlayer(new ComputerStrategyWeak());
		//Player playerX = new ComputerPlayer(new StudentStrategy());
		//Player playerW = new Player("human");
		//Player playerB = new ComputerPlayer(new DummyStrategy());
		//Player playerB = new ComputerPlayer(new ComputerStrategyWeak());
		
		Game game = new Game(playerW, playerB, 10, 10);
		this.game = game;
		squares = new Square[game.getHeight()][game.getWidth()];
		initialize();
		// change current player to start event handler
		game.start();
	}
}
